#!/usr/bin/env python
# coding: utf-8

# In[93]:


import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from datetime import datetime
from astropy import constants as const
from collections import Counter


# In[134]:


def query_parameters(system_name):
    
    # NASA Exoplanet Archive Downloader
    # inspired by https://github.com/ethankruse/exoplots

    """Downloads candidate and confirmed planet tables from NExSci"""


    NEW_API = 'https://exoplanetarchive.ipac.caltech.edu/TAP/sync?query='
    # The "exoplanets" table includes all confirmed planets and hosts in the
    # archive with parameters derived from a single, published reference


    # New confirmed planets
    print("Downloading all confirmed planets from NExSci's new table...")

    columns='pl_name,hostname,default_flag,sy_snum,sy_pnum,discoverymethod,disc_year,disc_facility,tran_flag,soltype,pl_controv_flag,pl_refname,pl_orbper,pl_orbpererr1,pl_orbpererr2,pl_orbperlim,pl_orbsmax,pl_orbsmaxerr1,pl_orbsmaxerr2,pl_orbsmaxlim,pl_rade,pl_radeerr1,pl_radeerr2,pl_radelim,pl_radj,pl_radjerr1,pl_radjerr2,pl_radjlim,pl_bmasse,pl_bmasseerr1,pl_bmasseerr2,pl_bmasselim,pl_bmassj,pl_bmassjerr1,pl_bmassjerr2,pl_bmassjlim,pl_bmassprov,pl_orbeccen,pl_orbeccenerr1,pl_orbeccenerr2,pl_orbeccenlim,pl_insol,pl_insolerr1,pl_insolerr2,pl_insollim,pl_eqt,pl_eqterr1,pl_eqterr2,pl_eqtlim,pl_orbincl,pl_orbinclerr1,pl_orbinclerr2,pl_orbincllim,ttv_flag,pl_trandur,pl_trandurerr1,pl_trandurerr2,pl_trandurlim,pl_ratdor,pl_ratdorerr1,pl_ratdorerr2,pl_ratdorlim,pl_ratror,pl_ratrorerr1,pl_ratrorerr2,pl_ratrorlim,pl_occdep,pl_occdeperr1,pl_occdeperr2,pl_occdeplim,st_refname,st_spectype,st_teff,st_tefferr1,st_tefferr2,st_tefflim,st_rad,st_raderr1,st_raderr2,st_radlim,st_mass,st_masserr1,st_masserr2,st_masslim,st_met,st_meterr1,st_meterr2,st_metlim,st_metratio,st_logg,st_loggerr1,st_loggerr2,st_logglim,sy_refname,rastr,ra,decstr,dec,sy_dist,sy_disterr1,sy_disterr2,sy_vmag,sy_vmagerr1,sy_vmagerr2,sy_jmag,sy_jmagerr1,sy_jmagerr2,sy_hmag,sy_hmagerr1,sy_hmagerr2,sy_kmag,sy_kmagerr1,sy_kmagerr2,sy_gaiamag,sy_gaiamagerr1,sy_gaiamagerr2,rowupdate,pl_pubdate,releasedate'


    print("Downloading default_flag=1")
    where1 = 'where+default_flag=1'
    full1 = NEW_API + 'select+' + columns + '+from+ps+' + where1 + '&format=csv'
    df = pd.read_csv(full1)
    df.to_csv('default_params1.csv')

    with open('last_update_time.txt', 'w') as ff:
        ff.write(str(datetime.now()))
    
    # read in default flag = 1 csv
    
    df1_filename = "default_params1.csv"
    
    # assign to pandas dataframe
    
    df1 = pd.read_csv(
        df1_filename,
        header=0
    ).iloc[:, 1:]
    
    # find planetary system
    
    system_parameters = df1.loc[df1['hostname'] == system_name]
    
    # return number of planets
    
    number_of_planets = system_parameters['sy_pnum'].values
    
    # get planetary radii
    
    planet_radius = system_parameters['pl_radj'].values
    
    planet_radius_array_without_nan = [x for x in planet_radius if str(x) != 'nan']
    
    if len(planet_radius_array_without_nan) == 0:
        return ValueError('No planetary radii available for this system.')
    
    # get orbital periods
    
    orbital_period = system_parameters['pl_orbper'].values
    
    orbital_period_without_nan = [x for x in orbital_period if str(x) != 'nan']
    
    if len(orbital_period_without_nan) == 0:
        return ValueError('No periods available for this system.')
    
    # get semi_major axis
    
    semi_major_axis = system_parameters['pl_orbsmax'].values
    
    semi_major_axis_without_nan = [x for x in semi_major_axis if str(x) != 'nan']
    
    if len(semi_major_axis_without_nan) == 0:
        return ValueError('No semi-major axis available for this system.')
    
    # get planetary masses
    
    planet_mass = system_parameters['pl_bmassj'].values
    
    planet_mass_without_nan = [x for x in planet_mass if str(x) != 'nan']
    
    if len(planet_mass_without_nan) == 0:
        return ValueError('No masses available for this system.')
    
    # get eccentricities
    
    eccentricity = system_parameters['pl_orbeccen'].values
    
    eccentricity_without_nan = [x for x in  eccentricity if str(x) != 'nan']
    
    if len(eccentricity_without_nan) == 0:
        return ValueError('No eccentricities available for this system.')
    
    # find most frequent stellar radius and report it
    
    stellar_radius_array = system_parameters['st_rad'].values
    
    stellar_radius_array_without_nan = [x for x in stellar_radius_array if str(x) != 'nan']
    
    if len(stellar_radius_array_without_nan) == 0:
        return ValueError('No stellar radius measurement available.')
    
    stellar_radius_max = [num for num, count in Counter(stellar_radius_array_without_nan).most_common(1)]
    
    stellar_radius = stellar_radius_max[0]
    
    # find most frequent stellar temp and report it
    
    stellar_temp_array = system_parameters['st_teff'].values # stellar temp array
    
    stellar_temp_array_without_nan = [x for x in stellar_temp_array if str(x) != 'nan']
    
    if len(stellar_temp_array_without_nan) == 0:
        return ValueError('No stellar temperature measurement available.')
    
    stellar_temp_max = [num for num, count in Counter(stellar_temp_array_without_nan).most_common(1)]
    
    stellar_temp = int(stellar_temp_max[0])
    
    return planet_radius, orbital_period, semi_major_axis, planet_mass, eccentricity, stellar_radius, stellar_temp, number_of_planets









import matplotlib.pyplot as plt
#from matplotlib.animation import FuncAnimation
import matplotlib.animation as an
import numpy as np


#defining function that makes an ellipse 
def ellipse(phi,a,e):
    b = np.sqrt(1-e**2) * a 
    return np.array([a*np.cos(phi), b*np.sin(phi)])

#defining a function that takes the temperature of a star and returns a color based on that 
def suncolor(teff):
    """ Sun color selector 

    Selects a color for plotting the star based on its temperature 

    Args: 
        teff (float): stellar temperature (K)

    Returns: 
        color (string): color in which to plot star 
        
    """

    if teff < 3500:
        return 'red'

    if (teff >= 3500) and (teff < 5000):
        return 'orange'
    
    if (teff >= 5000) and (teff < 8000):
        return 'yellow'
    
    if (teff >= 8000) and (teff < 15000):
        return 'white'
    
    if teff >= 15000:
        return 'blue'

#defining function to make a circle 
def circle(phi,r):
    return np.array([r*np.cos(phi), r*np.sin(phi)])

#defining the function which creates an animation based on planetary parameters
def animator(a_s,pers,rs,rstar,norbs,es,tstar,system,dir):
    """ Animation function 

    Generates an animation based on the exoplanetary system input 

    Args: 
        a_s (array): semimajor axes (AU) 
        pers (array): periods of planets 
        rs (array): planetary radii (R_jup)
        rstar (float): stellar radius (R_sun)
        norbs (int): number of orbits desired in animation 
        es (array): eccentricities of planets 
        tstar (float): temperature of star (K)
        system (string): system to make the animation of 
        dir (string): directory in which to save animation (default current directory)

        Note: units are not important, as all quantities will be scaled relative to each other. 
        As long as all values use the same units, any unit system works. 

    Returns: 
        Saved animation file (.gif)
        
    """

    #finding the number of planets based on the length of input 
    nplanets = len(a_s)

    #defining the size of the plot as 2 times the maximum semimajor axis
    a_plot = 1.5*np.max(a_s)

    #setting normalizing factors such that the innermost planet has an a of 1 and period of 1
    norma = np.min(a_s)
    normper = np.min(pers)

    #normalizing planet radii such that an earth radius planet has a size of 1 
    rearth = 6.37e8
    rjupiter = 7.05e9
    normr = rearth/rjupiter

    #creating the figure and setting bounds 
    fig, ax = plt.subplots()
    ax.set_aspect("equal")
    ax.set_xticks([])
    ax.set_yticks([])
    scale = 3*a_plot
    ax.set_xlabel("{:.1f}".format(scale) + " AU",loc='right')
    ax.set_facecolor('xkcd:black')
    ax.set_title(system,loc='left')
     
    ax.axis([-a_plot/norma,a_plot/norma,-a_plot/norma,a_plot/norma])

    #plotting the star as a point in the center, increasing star size
    sun = ax.plot(0,0, marker="o",markersize = 10*rstar,color=suncolor(tstar)) 


    #setting individual cases for each possible number of planets in the system
    if nplanets == 1: 
        
        #normalizing parameters of system 
        a1 = a_s[0]/norma
        per1 = pers[0]/normper
        r1 = rs[0]/normr

        #creating the plabet object and setting size and radius based on normalized parameters
        p1, = ax.plot(0,a1, marker="o",markersize = r1,color='xkcd:eggshell')

        #defining the function which updates the location of the planet throughout its elliptical orbit
        def update(phi):
            x1,y1 = ellipse(phi/per1,a1,es[0])
            p1.set_data([x1],[y1])
            return p1,

        #creating the animation and making it loop over 360 degrees of phi 
        ani = an.FuncAnimation(fig, update, interval=10, blit=True, repeat=True,
                        frames=np.linspace(0,norbs*2*np.pi,360, endpoint=False))
        #saving the animation 
        ani.save(dir+str(system)+'.gif')

        from IPython.display import HTML
        HTML(ani.to_jshtml())
 
    if nplanets == 2: 

        a1 = a_s[0]/norma
        per1 = pers[0]/normper
        r1 = rs[0]/normr
        a2 = a_s[1]/norma
        per2 = pers[1]/normper
        r2 = rs[1]/normr
        
        p1, = ax.plot(0,a1, marker="o",markersize = r1,color='xkcd:eggshell')
        p2, = ax.plot(0,a2, marker="o",markersize = r2,color='xkcd:eggshell')

        def update(phi):
            x1,y1 = ellipse(phi/per1,a1,es[0])
            x2,y2 = ellipse(phi/per2,a2,es[1])
            p1.set_data([x1],[y1])
            p2.set_data([x2],[y2])
            return p1,p2,

        ani = an.FuncAnimation(fig, update, interval=10, blit=True, repeat=True,
                        frames=np.linspace(0,norbs*2*np.pi,360, endpoint=False))
        ani.save(dir+str(system)+'.gif')

        from IPython.display import HTML
        HTML(ani.to_jshtml())

    if nplanets == 3: 

        a1 = a_s[0]/norma
        a2 = a_s[1]/norma
        a3 = a_s[2]/norma
        per1 = pers[0]/normper
        per2 = pers[1]/normper
        per3 = pers[2]/normper
        r1 = rs[0]/normr
        r2 = rs[1]/normr
        r3 = rs[2]/normr
        
        p1, = ax.plot(0,a1, marker="o",markersize = r1,color='xkcd:eggshell')
        p2, = ax.plot(0,a2, marker="o",markersize = r2,color='xkcd:eggshell')
        p3, = ax.plot(0,a3, marker="o",markersize = r3,color='xkcd:eggshell')

        def update(phi):
            x1,y1 = ellipse(phi/per1,a1,es[0])
            x2,y2 = ellipse(phi/per2,a2,es[1])
            x3,y3 = ellipse(phi/per3,a3,es[2])
            p1.set_data([x1],[y1])
            p2.set_data([x2],[y2])
            p3.set_data([x3],[y3])
            return p1,p2,p3,

        ani = an.FuncAnimation(fig, update, interval=10, blit=True, repeat=True,
                        frames=np.linspace(0,norbs*2*np.pi,360, endpoint=False))
        ani.save(dir+str(system)+'.gif')

        from IPython.display import HTML
        HTML(ani.to_jshtml())


    if nplanets == 4: 

        a1 = a_s[0]/norma
        a2 = a_s[1]/norma
        a3 = a_s[2]/norma
        a4 = a_s[3]/norma
        per1 = pers[0]/normper
        per2 = pers[1]/normper
        per3 = pers[2]/normper
        per4 = pers[3]/normper
        r1 = rs[0]/normr
        r2 = rs[1]/normr
        r3 = rs[2]/normr
        r4 = rs[3]/normr
        
        p1, = ax.plot(0,a1, marker="o",markersize = r1,color='xkcd:eggshell')
        p2, = ax.plot(0,a2, marker="o",markersize = r2,color='xkcd:eggshell')
        p3, = ax.plot(0,a3, marker="o",markersize = r3,color='xkcd:eggshell')
        p4, = ax.plot(0,a4, marker="o",markersize = r4,color='xkcd:eggshell')

        def update(phi):
            x1,y1 = ellipse(phi/per1,a1,es[0])
            x2,y2 = ellipse(phi/per2,a2,es[1])
            x3,y3 = ellipse(phi/per3,a3,es[2])
            x4,y4 = ellipse(phi/per4,a4,es[3])
            p1.set_data([x1],[y1])
            p2.set_data([x2],[y2])
            p3.set_data([x3],[y3])
            p4.set_data([x4],[y4])
            return p1,p2,p3,p4,

        ani = an.FuncAnimation(fig, update, interval=10, blit=True, repeat=True,
                        frames=np.linspace(0,norbs*2*np.pi,360, endpoint=False))
        ani.save(dir+str(system)+'.gif')

        from IPython.display import HTML
        HTML(ani.to_jshtml())

    if nplanets == 5: 

        a1 = a_s[0]/norma
        a2 = a_s[1]/norma
        a3 = a_s[2]/norma
        a4 = a_s[3]/norma
        a5 = a_s[4]/norma
        per1 = pers[0]/normper
        per2 = pers[1]/normper
        per3 = pers[2]/normper
        per4 = pers[3]/normper
        per5 = pers[4]/normper
        r1 = rs[0]/normr
        r2 = rs[1]/normr
        r3 = rs[2]/normr
        r4 = rs[3]/normr
        r5 = rs[4]/normr
        
        p1, = ax.plot(0,a1, marker="o",markersize = r1,color='xkcd:eggshell')
        p2, = ax.plot(0,a2, marker="o",markersize = r2,color='xkcd:eggshell')
        p3, = ax.plot(0,a3, marker="o",markersize = r3,color='xkcd:eggshell')
        p4, = ax.plot(0,a4, marker="o",markersize = r4,color='xkcd:eggshell')
        p5, = ax.plot(0,a5, marker="o",markersize = r5,color='xkcd:eggshell')

        def update(phi):
            x1,y1 = ellipse(phi/per1,a1,es[0])
            x2,y2 = ellipse(phi/per2,a2,es[1])
            x3,y3 = ellipse(phi/per3,a3,es[2])
            x4,y4 = ellipse(phi/per4,a4,es[3])
            x5,y5 = ellipse(phi/per5,a5,es[4])
            p1.set_data([x1],[y1])
            p2.set_data([x2],[y2])
            p3.set_data([x3],[y3])
            p4.set_data([x4],[y4])
            p5.set_data([x5],[y5])
            return p1,p2,p3,p4,p5,

        ani = an.FuncAnimation(fig, update, interval=10, blit=True, repeat=True,
                        frames=np.linspace(0,norbs*2*np.pi,360, endpoint=False))
        ani.save(dir+str(system)+'.gif')

        from IPython.display import HTML
        HTML(ani.to_jshtml())

    if nplanets == 6: 

        a1 = a_s[0]/norma
        a2 = a_s[1]/norma
        a3 = a_s[2]/norma
        a4 = a_s[3]/norma
        a5 = a_s[4]/norma
        a6 = a_s[5]/norma
        per1 = pers[0]/normper
        per2 = pers[1]/normper
        per3 = pers[2]/normper
        per4 = pers[3]/normper
        per5 = pers[4]/normper
        per6 = pers[5]/normper
        r1 = rs[0]/normr
        r2 = rs[1]/normr
        r3 = rs[2]/normr
        r4 = rs[3]/normr
        r5 = rs[4]/normr
        r6 = rs[5]/normr
        
        p1, = ax.plot(0,a1, marker="o",markersize = r1,color='xkcd:eggshell')
        p2, = ax.plot(0,a2, marker="o",markersize = r2,color='xkcd:eggshell')
        p3, = ax.plot(0,a3, marker="o",markersize = r3,color='xkcd:eggshell')
        p4, = ax.plot(0,a4, marker="o",markersize = r4,color='xkcd:eggshell')
        p5, = ax.plot(0,a5, marker="o",markersize = r5,color='xkcd:eggshell')
        p6, = ax.plot(0,a6, marker="o",markersize = r6,color='xkcd:eggshell')

        def update(phi):
            x1,y1 = ellipse(phi/per1,a1,es[0])
            x2,y2 = ellipse(phi/per2,a2,es[1])
            x3,y3 = ellipse(phi/per3,a3,es[2])
            x4,y4 = ellipse(phi/per4,a4,es[3])
            x5,y5 = ellipse(phi/per5,a5,es[4])
            x6,y6 = ellipse(phi/per6,a6,es[5])
            p1.set_data([x1],[y1])
            p2.set_data([x2],[y2])
            p3.set_data([x3],[y3])
            p4.set_data([x4],[y4])
            p5.set_data([x5],[y5])
            p6.set_data([x6],[y6])
            return p1,p2,p3,p4,p5,p6,

        ani = an.FuncAnimation(fig, update, interval=10, blit=True, repeat=True,
                        frames=np.linspace(0,norbs*2*np.pi,360, endpoint=False))
        ani.save(dir+str(system)+'.gif')

        from IPython.display import HTML
        HTML(ani.to_jshtml())

    if nplanets == 7: 

        a1 = a_s[0]/norma
        a2 = a_s[1]/norma
        a3 = a_s[2]/norma
        a4 = a_s[3]/norma
        a5 = a_s[4]/norma
        a6 = a_s[5]/norma
        a7 = a_s[6]/norma
        per1 = pers[0]/normper
        per2 = pers[1]/normper
        per3 = pers[2]/normper
        per4 = pers[3]/normper
        per5 = pers[4]/normper
        per6 = pers[5]/normper
        per7 = pers[6]/normper
        r1 = rs[0]/normr
        r2 = rs[1]/normr
        r3 = rs[2]/normr
        r4 = rs[3]/normr
        r5 = rs[4]/normr
        r6 = rs[5]/normr
        r7 = rs[6]/normr
        
        p1, = ax.plot(0,a1, marker="o",markersize = r1,color='xkcd:eggshell')
        p2, = ax.plot(0,a2, marker="o",markersize = r2,color='xkcd:eggshell')
        p3, = ax.plot(0,a3, marker="o",markersize = r3,color='xkcd:eggshell')
        p4, = ax.plot(0,a4, marker="o",markersize = r4,color='xkcd:eggshell')
        p5, = ax.plot(0,a5, marker="o",markersize = r5,color='xkcd:eggshell')
        p6, = ax.plot(0,a6, marker="o",markersize = r6,color='xkcd:eggshell')
        p7, = ax.plot(0,a7, marker="o",markersize = r7,color='xkcd:eggshell')

        def update(phi):
            x1,y1 = ellipse(phi/per1,a1,es[0])
            x2,y2 = ellipse(phi/per2,a2,es[1])
            x3,y3 = ellipse(phi/per3,a3,es[2])
            x4,y4 = ellipse(phi/per4,a4,es[3])
            x5,y5 = ellipse(phi/per5,a5,es[4])
            x6,y6 = ellipse(phi/per6,a6,es[5])
            x7,y7 = ellipse(phi/per7,a7,es[6])
            p1.set_data([x1],[y1])
            p2.set_data([x2],[y2])
            p3.set_data([x3],[y3])
            p4.set_data([x4],[y4])
            p5.set_data([x5],[y5])
            p6.set_data([x6],[y6])
            p7.set_data([x7],[y7])
            return p1,p2,p3,p4,p5,p6,p7,

        ani = an.FuncAnimation(fig, update, interval=10, blit=True, repeat=True,
                        frames=np.linspace(0,norbs*2*np.pi,360, endpoint=False))
        ani.save(str(system)+'.gif')

        from IPython.display import HTML
        HTML(dir+ani.to_jshtml())

    if nplanets == 8: 

        a1 = a_s[0]/norma
        a2 = a_s[1]/norma
        a3 = a_s[2]/norma
        a4 = a_s[3]/norma
        a5 = a_s[4]/norma
        a6 = a_s[5]/norma
        a7 = a_s[6]/norma
        a8 = a_s[7]/norma
        per1 = pers[0]/normper
        per2 = pers[1]/normper
        per3 = pers[2]/normper
        per4 = pers[3]/normper
        per5 = pers[4]/normper
        per6 = pers[5]/normper
        per7 = pers[6]/normper
        per8 = pers[7]/normper
        r1 = rs[0]/normr
        r2 = rs[1]/normr
        r3 = rs[2]/normr
        r4 = rs[3]/normr
        r5 = rs[4]/normr
        r6 = rs[5]/normr
        r7 = rs[6]/normr
        r8 = rs[7]/normr
        
        p1, = ax.plot(0,a1, marker="o",markersize = r1,color='xkcd:eggshell')
        p2, = ax.plot(0,a2, marker="o",markersize = r2,color='xkcd:eggshell')
        p3, = ax.plot(0,a3, marker="o",markersize = r3,color='xkcd:eggshell')
        p4, = ax.plot(0,a4, marker="o",markersize = r4,color='xkcd:eggshell')
        p5, = ax.plot(0,a5, marker="o",markersize = r5,color='xkcd:eggshell')
        p6, = ax.plot(0,a6, marker="o",markersize = r6,color='xkcd:eggshell')
        p7, = ax.plot(0,a7, marker="o",markersize = r7,color='xkcd:eggshell')
        p8, = ax.plot(0,a7, marker="o",markersize = r8,color='xkcd:eggshell')

        def update(phi):
            x1,y1 = ellipse(phi/per1,a1,es[0])
            x2,y2 = ellipse(phi/per2,a2,es[1])
            x3,y3 = ellipse(phi/per3,a3,es[2])
            x4,y4 = ellipse(phi/per4,a4,es[3])
            x5,y5 = ellipse(phi/per5,a5,es[4])
            x6,y6 = ellipse(phi/per6,a6,es[5])
            x7,y7 = ellipse(phi/per7,a7,es[6])
            x8,y8 = ellipse(phi/per7,a7,es[6])
            p1.set_data([x1],[y1])
            p2.set_data([x2],[y2])
            p3.set_data([x3],[y3])
            p4.set_data([x4],[y4])
            p5.set_data([x5],[y5])
            p6.set_data([x7],[y7])
            p7.set_data([x8],[y8])
            return p1,p2,p3,p4,p5,p6,p7,p8,

        ani = an.FuncAnimation(fig, update, interval=10, blit=True, repeat=True,
                        frames=np.linspace(0,norbs*2*np.pi,360, endpoint=False))
        ani.save(dir+str(system)+'.gif')

        from IPython.display import HTML
        HTML(ani.to_jshtml())